@@ -11,8 +11,10 @@ import matplotlib.pyplot as plt
 
 #load data from an excel spreadsheet
 def ExcelMatrix(namearq, namesheet, Lineini, Columini, columnquantity, linesquantity):
+    print('Loading data...')
     df = pd.read_excel(namearq, sheet_name=namesheet, header=None)
-    Matriz = df.iloc[Lineini-1:Lineini+linesquantity-1, Columini-1:Columini+columnquantity-1].values.astype(np.float64)    
+    Matriz = df.iloc[Lineini-1:Lineini+linesquantity-1, Columini-1:Columini+columnquantity-1].values.astype(np.float64)
+    print('Loading data = ok')
     return Matriz
 
 
@@ -33,14 +35,16 @@ def Statistics(Calculated, Targeted):
     for j in range(0,numevar):
         
         erroari = Calculated[:,j] - Targeted[:,j]
+        erroper = (( Calculated[:,j] - Targeted[:,j] ) / Targeted[:,j])*100
         coefcorrelacao = np.corrcoef(Targeted[:,j], Calculated[:,j])[1][0]
+        
         print ('Correlation coefficient, variable ' + str(j+1) +' = ' + str(coefcorrelacao))
         print('')
-        print ('Mean difference, variable ' + str(j+1) +' = ' + str(np.mean(erroari)) + ' ')
+        print ('Mean difference, variable ' + str(j+1) +' = ' + str(np.mean(erroari)) + ' (' + str(np.mean(erroper)) + '%)')
         print('')
-        print ('Standard deviation, variable ' + str(j+1) +' = ' + str(np.std(erroari)) + '')
+        print ('Standard deviation, variable ' + str(j+1) +' = ' + str(np.std(erroari)) + ' (' + str(np.std(erroper)) + '%)')
         print('')
-        print ('Biggest absolute point difference, variable ' + str(j+1) +' = ' + str(np.max(abs(erroari))) + ' ')
+        print ('Biggest absolute point difference, variable ' + str(j+1) +' = ' + str(np.max(abs(erroari))) + ' ('  + str(np.max(abs(erroper))) + '%)' )
         print('------------------------')
 
 
@@ -49,7 +53,7 @@ def Statistics(Calculated, Targeted):
 
 
 
-#Plot two variables differences in a sequential series
+#Plot two variables ifferences in a sequential series
 def PlotComparative(Calculated, Targeted):
 
     numdata,numevar = Calculated.shape
@@ -70,7 +74,8 @@ def PlotComparative(Calculated, Targeted):
 
 
 
-#Plot two variables diferenças in a sequential series
+
+#Plot the differences of two variables in a sequential series
 def PlotComparative2(Calculated, Targeted):
 
     numdata,numevar = Calculated.shape
@@ -112,6 +117,37 @@ def PlotCorrelation(Calculated, Targeted):
 
 
 
+#Plota histograma do erro %
+def PlotHisto(Calculated, Targeted):
+
+    numdata,numevar = Calculated.shape
+
+    for j in range(0,numevar):    
+        eixoY1= Targeted[:,j]
+        eixoY2= Calculated[:,j]
+        eixoY3 = 100*(eixoY2 - eixoY1)/eixoY1 #erro percentual 0-100%
+
+        fig, ax = plt.subplots()
+        
+        desviopad = np.std(eixoY3) #devio padrão do erro
+        erromedio = np.mean(eixoY3) #erro medio
+        
+        textstr = '\n'.join((r'$\mathrm{average}=%.2f$' % (erromedio, ),r'$\sigma=%.2f$' % (desviopad, )))
+
+        ax.hist(eixoY3, bins=20, rwidth=0.9, color='blue', alpha=0.7, edgecolor='black')
+        # these are matplotlib.patch.Patch properties
+        props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
+
+        # place a text box in upper left in axes coords
+        ax.text(0.05, 0.95, textstr, transform=ax.transAxes, fontsize=14, verticalalignment='top', bbox=props)
+
+        plt.title('Percent error histogram', fontsize=15)
+        plt.xlabel('Variable ' + str(j+1) + ', %', fontsize=12)
+        plt.ylabel('Frequency', fontsize=12)
+
+        plt.show()
+
+
 
 
 
@@ -119,7 +155,7 @@ def PlotCorrelation(Calculated, Targeted):
 class nnet4:
 
     #global variables-------------------------------------------------------------------------
-    def __init__ (self, norma=2, coef=0, nn1c=1, nn2c=5, nn3c=5, nn4c=1, rate=0.01, epochs=1000, fa2c=5, fa3c=5, fa4c=0):
+    def __init__ (self, norma=2, coef=0, normout=1, nn1c=1, nn2c=5, nn3c=5, nn4c=1, rate=0.01, epochs=1000, fa2c=5, fa3c=5, fa4c=0):
         
         self.nn1c=nn1c
         self.nn2c=nn2c
@@ -131,7 +167,8 @@ class nnet4:
         self.fa3c=fa3c
         self.fa4c=fa4c
         self.norma=norma
-        self.coef=coef        
+        self.coef=coef
+        self.normout=normout
 
 
 
@@ -301,11 +338,18 @@ class nnet4:
             y = np.log( 1/(1+np.exp(-x)) )
             return y
 
+        if (typee==13): #x^2 (inf,inf)
+            y = x*x
+            return y
 
+        if (typee==14): #x^3 (inf,inf)
+            y = x*x*x
+            return y
 
-
-
-
+        if (typee==15): # Simetric Rectified Linear (inf,inf)
+            y2 = np.where(x > 1, x*0.25, x)
+            y = np.where(y2 < -1, y2*0.25, y2)
+            return y
 
 
 
@@ -366,9 +410,18 @@ class nnet4:
             y = np.exp(-x)/(1+np.exp(-x))
             return y
 
+        if (typee==13): #x^2 (inf,inf)
+            y = 2*x
+            return y
 
+        if (typee==14): #x^3 (inf,inf)
+            y = 3*x*x
+            return y
 
-
+        if (typee==15): # Simetric Rectified Linear (inf,inf)
+            y2 = np.where(x > 1, 0.25, 1)
+            y = np.where(y2 < -1, 0.25, 1)
+            return y
 
 
 
@@ -481,8 +534,9 @@ class nnet4:
 
 
         #Normaliza os parametros-------------------------
-        matX = self.Normalize(matX,"vmax_in.txt","vmin_in.txt","media_in.txt","desvio_in.txt")        
-        matY = self.Normalize(matY,"vmax_out.txt","vmin_out.txt","media_out.txt","desvio_out.txt")
+        matX = self.Normalize(matX,"vmax_in.txt","vmin_in.txt","media_in.txt","desvio_in.txt")
+        if (self.normout==1):
+            matY = self.Normalize(matY,"vmax_out.txt","vmin_out.txt","media_out.txt","desvio_out.txt")
 
 
         #Processo de aprendizado------------------------------------------------------------
@@ -727,8 +781,9 @@ class nnet4:
 
 
         #Normaliza os parametros-------------------------
-        matX = self.Normalize(matX,"vmax_in.txt","vmin_in.txt","media_in.txt","desvio_in.txt")         
-        matY = self.Normalize(matY,"vmax_out.txt","vmin_out.txt","media_out.txt","desvio_out.txt") 
+        matX = self.Normalize(matX,"vmax_in.txt","vmin_in.txt","media_in.txt","desvio_in.txt")
+        if (self.normout==1):
+            matY = self.Normalize(matY,"vmax_out.txt","vmin_out.txt","media_out.txt","desvio_out.txt") 
 
 
         #Processo de aprendizado------------------------------------------------------------
@@ -919,7 +974,8 @@ class nnet4:
             # saida----------------------------
             Calculated[caso][:] = c4[:]
 
-        Calculated = self.DesNormalize(Calculated, "vmax_out.txt", "vmin_out.txt","media_out.txt","desvio_out.txt") 
+        if (self.normout==1):            
+            Calculated = self.DesNormalize(Calculated, "vmax_out.txt", "vmin_out.txt","media_out.txt","desvio_out.txt") 
 
         return Calculated
 
